package com.mortals.xhx.module.converge.service.impl;
import com.mortals.framework.common.Rest;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.feign.device.IDeviceFeign;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.converge.model.*;
import com.mortals.xhx.module.converge.model.vo.AppVO;
import com.mortals.xhx.module.converge.model.vo.BranchVO;
import com.mortals.xhx.module.converge.model.vo.EquipmentVO;
import com.mortals.xhx.module.converge.model.vo.InterfaceStatsVO;
import com.mortals.xhx.module.converge.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.converge.dao.ConvergeSiteDao;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
* ConvergeSiteService
* 网点信息 service实现
*
* @author zxfei
* @date 2023-06-12
*/
@Service("convergeSiteService")
@Slf4j
public class ConvergeSiteServiceImpl extends AbstractCRUDServiceImpl<ConvergeSiteDao, ConvergeSiteEntity, Long> implements ConvergeSiteService {

    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private ConvergeDeviceService convergeDeviceService;
    @Autowired
    private ConvergeAppsService convergeAppsService;
    @Autowired
    private ConvergeAppsInterfaceService convergeAppsInterfaceService;
    @Autowired
    private ConvergeAppsInfoService convergeAppsInfoService;

    @Override
    public List<BranchVO> getConvergeData() throws AppException {
        List<BranchVO> result = new ArrayList<>();
        List<ConvergeSiteEntity> siteList = this.find(new ConvergeSiteQuery());
        if(CollectionUtils.isNotEmpty(siteList)){
//            SimpleDateFormat format=new SimpleDateFormat("yyyy-MM-dd");
//            Calendar calendar=Calendar.getInstance();
//            calendar.set(Calendar.DAY_OF_MONTH, 1);
//            String currMonthStart = format.format(calendar.getTime());
//            calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
//            String currMonthEnd = format.format(calendar.getTime());
//            String currMonth = DateUtils.getCurrDateTime("yyyy-MM");
            List<ConvergeAppsInfoEntity> appsInfoList = convergeAppsInfoService.getAllList();
            Map<String,String> appsInfoMap = new HashMap<>();
            if(CollectionUtils.isNotEmpty(appsInfoList)) {
                appsInfoList.forEach(i -> {
                    appsInfoMap.put(i.getAppEname(), i.getAppName());
                });
            }
            for(ConvergeSiteEntity siteEntity:siteList){
                BranchVO branchVO = new BranchVO();
                BeanUtils.copyProperties(siteEntity,branchVO,BeanUtil.getNullPropertyNames(siteEntity));
                List<ConvergeDeviceEntity> deviceEntityList = convergeDeviceService.find(new ConvergeDeviceQuery().siteId(siteEntity.getSiteId()));
                if(CollectionUtils.isNotEmpty(deviceEntityList)){
                    List<EquipmentVO> equipments = new ArrayList<>();
                    for (ConvergeDeviceEntity deviceEntity:deviceEntityList){
                        EquipmentVO equipmentVO = new EquipmentVO();
                        BeanUtils.copyProperties(deviceEntity,equipmentVO,BeanUtil.getNullPropertyNames(deviceEntity));
                        ConvergeAppsQuery appsQuery = new ConvergeAppsQuery();
                        appsQuery.setSiteId(deviceEntity.getSiteId());
                        appsQuery.setEquipmentId(deviceEntity.getId());
//                        appsQuery.setStatYearMonth(currMonth);
                        List<ConvergeAppsEntity> apps = convergeAppsService.find(appsQuery);
                        if(CollectionUtils.isEmpty(apps)){
                            continue;
                        }
                        for(ConvergeAppsEntity item:apps){
                            ConvergeAppsInterfaceEntity query = new ConvergeAppsInterfaceEntity();
                            query.setEcode(deviceEntity.getLogo());
                            query.setApplicationId(item.getAppEname());
//                            query.setCurrMonthStart(currMonthStart);
//                            query.setCurrMonthEnd(currMonthEnd);
                            List<InterfaceStatsVO> interfaceEntityList = convergeAppsInterfaceService.getTotalInvokeSum(query);
                            if(CollectionUtils.isEmpty(interfaceEntityList) || interfaceEntityList.get(0) == null){
                                continue;
                            }
                            item.setInterfaceEntityList(interfaceEntityList);
                        }
                        equipmentVO.setAppStats(convertAppVO(apps,appsInfoMap));
                        equipmentVO.setOpr("U");
                        if(CollectionUtils.isEmpty(equipmentVO.getAppStats())){
                            continue;
                        }
                        equipments.add(equipmentVO);
                    }
                    branchVO.setEquipments(equipments);
                    branchVO.setOpr("U");
                }else {
                    //branchVO.setEquipments(Collections.emptyList());
                    continue;
                }
                result.add(branchVO);

            }
        }
        return result;
    }

    @Override
    public void doConvergeSite() throws AppException {
        SitePdu query = new SitePdu();
        query.setSize(-1);
        Rest<RespData<List<SitePdu>>> result = siteFeign.list(query);
        List<ConvergeSiteEntity> siteList = new ArrayList<>();
        if(result.getCode() > 0) {
            for(SitePdu sitePdu:result.getData().getData()){
                ConvergeSiteEntity convergeSiteEntity = convertBySitePdu(sitePdu);
                siteList.add(convergeSiteEntity);
            }
        }
        for(ConvergeSiteEntity site:siteList){
            ConvergeSiteEntity temp = this.selectOne(new ConvergeSiteQuery().branchLogo(site.getBranchLogo()));
            if(temp == null){
                site.setOpr("A");
                this.save(site);
            }else {
                if(equals(temp,site)){
                    site.setOpr("P");
                }else {
                    site.setOpr("U");
                }
                site.setId(temp.getId());
                this.update(site);
            }
        }

    }

    private ConvergeSiteEntity convertBySitePdu(SitePdu sitePdu){
        ConvergeSiteEntity entity = new ConvergeSiteEntity();
        entity.setSiteId(sitePdu.getId());
        entity.setBranchName(sitePdu.getSiteName());
        entity.setBranchLogo(sitePdu.getSiteCode());
        entity.setStatus("Y");
        entity.setServerTime("星期一至星期五 上午：09:00-12:00 下午：14:00-17:00 备注：法定节假日除外;");
        entity.setConsultPhone(sitePdu.getSiteTel());
        entity.setBranchStyle("1");
        entity.setAddress(sitePdu.getDetailAddress());
        return entity;
    }


    private boolean equals(ConvergeSiteEntity temp,ConvergeSiteEntity entity){
        if(!temp.getAddress().equals(entity.getAddress())){
            return false;
        }
        if(!temp.getBranchLogo().equals(entity.getBranchLogo())){
            return false;
        }
        if(!temp.getBranchName().equals(entity.getBranchName())){
            return false;
        }
        if(!temp.getStatus().equals(entity.getStatus())){
            return false;
        }
        if(!temp.getConsultPhone().equals(entity.getConsultPhone())){
            return false;
        }
        return true;
    }

    private List<AppVO> convertAppVO(List<ConvergeAppsEntity> apps,Map<String,String> appsInfoMap){
        if(CollectionUtils.isEmpty(apps)){
            return Collections.emptyList();
        }else {
            List<AppVO> voList = new ArrayList<>();
            for (ConvergeAppsEntity item:apps){
                if(CollectionUtils.isEmpty(item.getInterfaceEntityList()) || item.getInterfaceEntityList().get(0) == null){
                    continue;
                }
                AppVO vo = new AppVO();
                BeanUtils.copyProperties(item,vo,BeanUtil.getNullPropertyNames(item));
                if(appsInfoMap.containsKey(item.getAppEname())){
                    vo.setAppName(appsInfoMap.get(item.getAppEname()));
                    vo.setAppSimpleName(appsInfoMap.get(item.getAppEname()));
                }
                vo.setInterfaceStats(item.getInterfaceEntityList());
                voList.add(vo);
            }
            return voList;
        }
    }
}